@@ -21,23 +21,33 @@ package org.apache.synapse.core.axis2;
 
 import org.apache.axiom.om.OMElement;
 import org.apache.axiom.om.OMNamespace;
+import org.apache.axiom.om.OMNode;
 import org.apache.axis2.AxisFault;
 import org.apache.axis2.description.*;
 import org.apache.axis2.engine.AxisConfiguration;
+import org.apache.axis2.engine.AxisEvent;
+import org.apache.axis2.util.JavaUtils;
+import org.apache.axis2.wsdl.WSDLConstants;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.neethi.Policy;
 import org.apache.neethi.PolicyEngine;
+import org.apache.synapse.SynapseArtifact;
 import org.apache.synapse.SynapseConstants;
 import org.apache.synapse.SynapseException;
+import org.apache.synapse.aspects.AspectConfigurable;
+import org.apache.synapse.aspects.AspectConfiguration;
 import org.apache.synapse.config.SynapseConfigUtils;
 import org.apache.synapse.config.SynapseConfiguration;
 import org.apache.synapse.core.SynapseEnvironment;
 import org.apache.synapse.endpoints.Endpoint;
 import org.apache.synapse.mediators.base.SequenceMediator;
+import org.apache.synapse.util.PolicyInfo;
+import org.apache.synapse.util.resolver.CustomWSDLLocator;
+import org.apache.synapse.util.resolver.CustomXmlSchemaURIResolver;
+import org.apache.synapse.util.resolver.ResourceMap;
 import org.xml.sax.InputSource;
 
-import javax.xml.namespace.QName;
 import javax.xml.stream.XMLStreamException;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
@@ -64,18 +74,23 @@ import java.util.*;
  *    </publishWSDL>?
  *    <enableSec/>?
  *    <enableRM/>?
- *    <policy key="string">?
+ *    <policy key="string" [type=("in" |"out")] [operationName="string"]
+ *      [operationNamespace="string"]>?
  *       // optional service parameters
  *    <parameter name="string">
  *       text | xml
  *    </parameter>?
  * </proxy-service>
  */
-public class ProxyService {
+@SuppressWarnings({"UnusedDeclaration"})
+public class ProxyService implements AspectConfigurable, SynapseArtifact {
 
     private static final Log log = LogFactory.getLog(ProxyService.class);
     private static final Log trace = LogFactory.getLog(SynapseConstants.TRACE_LOGGER);
     private final Log serviceLog;
+
+    public static final String ABSOLUTE_SCHEMA_URL_PARAM = "showAbsoluteSchemaURL";
+    
     /**
      * The name of the proxy service
      */
@@ -117,17 +132,17 @@ public class ProxyService {
      */
     private SequenceMediator targetInLineInSequence = null;
     /**
-     * The inlined definition of the target out-sequence, if defined
+     * The in-lined definition of the target out-sequence, if defined
      */
     private SequenceMediator targetInLineOutSequence = null;
     /**
-     * The inlined definition of the target fault-sequence, if defined
+     * The in-lined definition of the target fault-sequence, if defined
      */
     private SequenceMediator targetInLineFaultSequence = null;
     /**
      * A list of any service parameters (e.g. JMS parameters etc)
      */
-    private Map parameters = new HashMap();
+    private final Map<String, Object> parameters = new HashMap<String, Object>();
     /**
      * The key for the base WSDL
      */
@@ -137,7 +152,7 @@ public class ProxyService {
      */
     private URI wsdlURI;
     /**
-     * The inlined representation of the service WSDL, if defined inline
+     * The in-lined representation of the service WSDL, if defined inline
      */
     private Object inLineWSDL;
     /**
@@ -145,10 +160,27 @@ public class ProxyService {
      * by the service WSDL to be located in the registry.
      */
     private ResourceMap resourceMap;
+    /**
+     * Policies to be set to the service, this can include service level, operation level,
+     * message level or hybrid level policies as well.
+     */
+    private List<PolicyInfo> policies = new ArrayList<PolicyInfo>();
     /**
      * The keys for any supplied policies that would apply at the service level
      */
-    private List serviceLevelPolicies = new ArrayList();
+    private final List<String> serviceLevelPolicies = new ArrayList<String>();
+    /**
+     * The keys for any supplied policies that would apply at the in message level
+     */
+    private List<String> inMessagePolicies = new ArrayList<String>();
+    /**
+     * The keys for any supplied policies that would apply at the out message level
+     */
+    private List<String> outMessagePolicies = new ArrayList<String>();
+    /**
+     * Should WS Addressing be engaged on this service
+     */
+    private boolean wsAddrEnabled = false;
     /**
      * Should WS RM be engaged on this service
      */
@@ -167,16 +199,17 @@ public class ProxyService {
     private boolean running = false;
 
     public static final String ALL_TRANSPORTS = "all";
-
-    /**
-     * To decide to whether statistics should have collected or not
-     */
-    private int statisticsState = SynapseConstants.STATISTICS_UNSET;
+   
     /**
      * The variable that indicate tracing on or off for the current mediator
      */
-    protected int traceState = SynapseConstants.TRACING_UNSET;
+    private int traceState = SynapseConstants.TRACING_UNSET;
 
+    private AspectConfiguration aspectConfiguration;
+
+    private String fileName;
+
+    private String serviceGroup;
 
     /**
      * Constructor
@@ -191,7 +224,7 @@ public class ProxyService {
     /**
      * Build the underlying Axis2 service from the Proxy service definition
      *
-     * @param synCfg the Synapse configuration
+     * @param synCfg  the Synapse configuration
      * @param axisCfg the Axis2 configuration
      * @return the Axis2 service for the Proxy
      */
@@ -209,6 +242,12 @@ public class ProxyService {
 
         InputStream wsdlInputStream = null;
         OMElement wsdlElement = null;
+        boolean wsdlFound = false;
+        String publishWSDL = null;
+
+        SynapseEnvironment synEnv = SynapseConfigUtils.getSynapseEnvironment(axisCfg);
+        String synapseHome = synEnv != null ? synEnv.getServerContextInformation()
+                .getServerConfigurationInformation().getSynapseHome() : "";
 
         if (wsdlKey != null) {
             synCfg.getEntryDefinition(wsdlKey);
@@ -216,17 +255,74 @@ public class ProxyService {
             if (keyObject instanceof OMElement) {
                 wsdlElement = (OMElement) keyObject;
             }
+            wsdlFound = true;
         } else if (inLineWSDL != null) {
             wsdlElement = (OMElement) inLineWSDL;
+            wsdlFound = true;
         } else if (wsdlURI != null) {
             try {
-                URL url = wsdlURI.toURL();
-                wsdlElement = SynapseConfigUtils.getOMElementFromURL(url.toString());
+            	URL url = wsdlURI.toURL();
+                publishWSDL = url.toString();
+
+                OMNode node = SynapseConfigUtils.getOMElementFromURL(publishWSDL, synapseHome);
+                if (node instanceof OMElement) {
+                    wsdlElement = (OMElement) node;
+                }
+                wsdlFound = true;
             } catch (MalformedURLException e) {
                 handleException("Malformed URI for wsdl", e);
             } catch (IOException e) {
-                handleException("Error reading from wsdl URI", e);
+                //handleException("Error reading from wsdl URI", e);
+                boolean enablePublishWSDLSafeMode = false;
+                Map proxyParameters= this.getParameterMap();
+                if (!proxyParameters.isEmpty()) {
+                    if (proxyParameters.containsKey("enablePublishWSDLSafeMode")) {
+                        enablePublishWSDLSafeMode =
+                                Boolean.parseBoolean(
+                                        proxyParameters.get("enablePublishWSDLSafeMode").
+                                                toString().toLowerCase());
+                    } else {
+                        if (trace()) {
+                            trace.info("WSDL was unable to load for: " + publishWSDL);
+                            trace.info("Please add <syn:parameter name=\"enableURISafeMode\">true" +
+                                    "</syn:parameter> to proxy service.");
+                        }
+                        handleException("Error reading from wsdl URI", e);
+                    }
+                }
+
+                if (enablePublishWSDLSafeMode) {
+                    // this is if the wsdl cannot be loaded... create a dummy service and an operation for which
+                    // our SynapseDispatcher will properly dispatch to
+
+                    //!!!Need to add a reload function... And display that the wsdl/service is offline!!!
+                    if (trace()) {
+                        trace.info("WSDL was unable to load for: " + publishWSDL);
+                        trace.info("enableURISafeMode: true");
+                    }
+
+                    log.warn("Unable to load the WSDL for : " + name, e);
+                    return null;
+                } else {
+                    if (trace()) {
+                        trace.info("WSDL was unable to load for: " + publishWSDL);
+                        trace.info("enableURISafeMode: false");
+                    }
+
+                    handleException("Error reading from wsdl URI", e);
+                }
             }
+        } else {
+            // this is for POX... create a dummy service and an operation for which
+            // our SynapseDispatcher will properly dispatch to
+            if (trace()) trace.info("Did not find a WSDL. Assuming a POX or Legacy service");
+            proxyService = new AxisService();
+            AxisOperation mediateOperation = new InOutAxisOperation(
+                    SynapseConstants.SYNAPSE_OPERATION_NAME);
+            // Set the names of the two messages so that Axis2 is able to produce a WSDL (see SYNAPSE-366):
+            mediateOperation.getMessage(WSDLConstants.MESSAGE_LABEL_IN_VALUE).setName("in");
+            mediateOperation.getMessage(WSDLConstants.MESSAGE_LABEL_OUT_VALUE).setName("out");
+            proxyService.addOperation(mediateOperation);
         }
 
         // if a WSDL was found
@@ -251,7 +347,6 @@ public class ProxyService {
                         + wsdlNamespace.getNamespaceURI());
 
                     if (wsdlNamespace != null) {
-                        boolean isWSDL11 = false;
                         WSDLToAxisServiceBuilder wsdlToAxisServiceBuilder = null;
 
                         if (WSDL2Constants.WSDL_NAMESPACE.
@@ -263,7 +358,6 @@ public class ProxyService {
                                 equals(wsdlNamespace.getNamespaceURI())) {
                             wsdlToAxisServiceBuilder =
                                     new WSDL11ToAxisServiceBuilder(wsdlInputStream);
-                            isWSDL11 = true;
                         } else {
                             handleException("Unknown WSDL format.. not WSDL 1.1 or WSDL 2.0");
                         }
@@ -273,8 +367,8 @@ public class ProxyService {
                                     "Could not get the WSDL to Axis Service Builder");
                         }
 
-                        wsdlToAxisServiceBuilder.setBaseUri(
-                                wsdlURI != null ? wsdlURI.toString() : "");
+                        wsdlToAxisServiceBuilder.setBaseUri(wsdlURI != null ?
+                                wsdlURI.toString() : synapseHome);
 
                         if (trace()) {
                             trace.info("Setting up custom resolvers");
@@ -284,11 +378,11 @@ public class ProxyService {
                         if (resourceMap != null) {
                             // if the resource map is available use it
                             wsdlToAxisServiceBuilder.setCustomResolver(
-                                new CustomURIResolver(resourceMap, synCfg));
+                                new CustomXmlSchemaURIResolver(resourceMap, synCfg));
                             // Axis 2 also needs a WSDLLocator for WSDL 1.1 documents
                             if (wsdlToAxisServiceBuilder instanceof WSDL11ToAxisServiceBuilder) {
                                 ((WSDL11ToAxisServiceBuilder)
-                                    wsdlToAxisServiceBuilder).setCustomWSLD4JResolver(
+                                    wsdlToAxisServiceBuilder).setCustomWSDLResolver(
                                     new CustomWSDLLocator(new InputSource(wsdlInputStream),
                                                           wsdlURI != null ? wsdlURI.toString() : "",
                                                           resourceMap, synCfg));
@@ -297,13 +391,13 @@ public class ProxyService {
                             //if the resource map isn't available ,
                             //then each import URIs will be resolved using base URI 
                             wsdlToAxisServiceBuilder.setCustomResolver(
-                                new CustomURIResolver());
+                                new CustomXmlSchemaURIResolver());
                             // Axis 2 also needs a WSDLLocator for WSDL 1.1 documents
                             if (wsdlToAxisServiceBuilder instanceof WSDL11ToAxisServiceBuilder) {
                                 ((WSDL11ToAxisServiceBuilder)
-                                    wsdlToAxisServiceBuilder).setCustomWSLD4JResolver(
+                                    wsdlToAxisServiceBuilder).setCustomWSDLResolver(
                                     new CustomWSDLLocator(new InputSource(wsdlInputStream),
-                                                          wsdlURI != null ? wsdlURI.toString() : ""));
+                                            wsdlURI != null ? wsdlURI.toString() : ""));
                             }
                         }
 
@@ -314,25 +408,12 @@ public class ProxyService {
                             }
                         }
                         proxyService = wsdlToAxisServiceBuilder.populateService();
-                        proxyService.setWsdlFound(true);
 
-                        if (isWSDL11) {
-                            // workaround to support WSDL 2.0 generation when only a WSDL 1.1
-                            // is supplied
-                            Collection endpoints = proxyService.getEndpoints().values();
-                            Iterator iter = endpoints.iterator();
-                            while (iter.hasNext()) {
-                                AxisEndpoint endpoint = (AxisEndpoint) iter.next();
-                                Iterator children = endpoint.getBinding().getChildren();
-                                while (children.hasNext()) {
-                                    AxisBindingOperation axisBindingOperation =
-                                        (AxisBindingOperation) children.next();
-                                    axisBindingOperation.setProperty(
-                                        WSDL2Constants.ATTR_WHTTP_IGNORE_UNCITED,
-                                        new Boolean(false));
-                                }
-                            }
-                        }
+                        // this is to clear the bindings and ports already in the WSDL so that the
+                        // service will generate the bindings on calling the printWSDL otherwise
+                        // the WSDL which will be shown is same as the original WSDL except for the
+                        // service name
+                        proxyService.getEndpoints().clear();
 
                     } else {
                         handleException("Unknown WSDL format.. not WSDL 1.1 or WSDL 2.0");
@@ -344,13 +425,9 @@ public class ProxyService {
                     handleException("Error reading WSDL", ioe);
                 }
             }
-        } else {
-            // this is for POX... create a dummy service and an operation for which
-            // our SynapseDispatcher will properly dispatch to
-            if (trace()) trace.info("Did not find a WSDL. Assuming a POX or Legacy service");
-            proxyService = new AxisService();
-            AxisOperation mediateOperation = new InOutAxisOperation(new QName("mediate"));
-            proxyService.addOperation(mediateOperation);
+        } else if (wsdlFound) {
+            handleException("Couldn't build the proxy service : " + name
+                    + ". Unable to locate the specified WSDL to build the service");
         }
 
         // Set the name and description. Currently Axis2 uses the name as the
@@ -377,9 +454,8 @@ public class ProxyService {
         if (trace() && parameters.size() > 0) {
             trace.info("Setting service parameters : " + parameters);
         }
-        Iterator iter = parameters.keySet().iterator();
-        while (iter.hasNext()) {
-            String name = (String) iter.next();
+        for (Object o : parameters.keySet()) {
+            String name = (String) o;
             Object value = parameters.get(name);
 
             Parameter p = new Parameter();
@@ -390,42 +466,70 @@ public class ProxyService {
                 proxyService.addParameter(p);
             } catch (AxisFault af) {
                 handleException("Error setting parameter : " + name + "" +
-                        "to proxy service as a Parameter", af);
+                    "to proxy service as a Parameter", af);
             }
         }
 
-        // if service level policies are specified, apply them
-        if (!serviceLevelPolicies.isEmpty()) {
+        if (JavaUtils.isTrueExplicitly(proxyService.getParameterValue(ABSOLUTE_SCHEMA_URL_PARAM))) {
+            proxyService.setCustomSchemaNamePrefix("");
+        }
 
-            if (trace()) {
-                trace.info("Setting service level policies : " + serviceLevelPolicies);
-            }
-            Policy svcEffectivePolicy = null;
-            iter = serviceLevelPolicies.iterator();
+        if (!policies.isEmpty()) {
+
+            for (PolicyInfo pi : policies) {
+
+                if (pi.isServicePolicy()) {
+
+                    proxyService.getPolicySubject().attachPolicy(
+                            getPolicyFromKey(pi.getPolicyKey(), synCfg));
+
+                } else if (pi.isOperationPolicy()) {
+
+                    AxisOperation op = proxyService.getOperation(pi.getOperation());
+                    if (op != null) {
+                        op.getPolicySubject().attachPolicy(
+                                getPolicyFromKey(pi.getPolicyKey(), synCfg));
 
-            while (iter.hasNext()) {
-                String policyKey = (String) iter.next();
-                synCfg.getEntryDefinition(policyKey);
-                Object policyProp = synCfg.getEntry(policyKey);
-                if (policyProp != null) {
-                    if (svcEffectivePolicy == null) {
-                        svcEffectivePolicy = PolicyEngine.getPolicy(
-                            SynapseConfigUtils.getStreamSource(policyProp).getInputStream());
                     } else {
-                        svcEffectivePolicy = svcEffectivePolicy.merge(
-                            PolicyEngine.getPolicy(SynapseConfigUtils.getStreamSource(
-                                policyProp).getInputStream()));
+                        handleException("Couldn't find the operation specified " +
+                                "by the QName : " + pi.getOperation());
                     }
-                }
-            }
-            PolicyInclude pi = proxyService.getPolicyInclude();
-            if (pi != null && svcEffectivePolicy != null) {
-                if (trace()) {
-                    if (trace.isTraceEnabled()) {
-                        trace.trace("Effective policy applied : " + svcEffectivePolicy);
+
+                } else if (pi.isMessagePolicy()) {
+
+                    if (pi.getOperation() != null) {
+
+                        AxisOperation op = proxyService.getOperation(pi.getOperation());
+                        if (op != null) {
+                            op.getMessage(pi.getMessageLable()).getPolicySubject().attachPolicy(
+                                    getPolicyFromKey(pi.getPolicyKey(), synCfg));
+                        } else {
+                            handleException("Couldn't find the operation " +
+                                    "specified by the QName : " + pi.getOperation());
+                        }
+
+                    } else {
+                        // operation is not specified and hence apply to all the applicable messages
+                        for (Iterator itr = proxyService.getOperations(); itr.hasNext();) {
+                            Object obj = itr.next();
+                            if (obj instanceof AxisOperation) {
+                                // check whether the policy is applicable
+                                if (!((obj instanceof OutOnlyAxisOperation && pi.getType()
+                                        == PolicyInfo.MESSAGE_TYPE_IN) ||
+                                        (obj instanceof InOnlyAxisOperation
+                                        && pi.getType() == PolicyInfo.MESSAGE_TYPE_OUT))) {
+
+                                    AxisMessage message = ((AxisOperation)
+                                            obj).getMessage(pi.getMessageLable());
+                                    message.getPolicySubject().attachPolicy(
+                                            getPolicyFromKey(pi.getPolicyKey(), synCfg));
+                                }
+                            }
+                        }
                     }
+                } else {
+                    handleException("Undefined Policy type");
                 }
-                pi.addPolicyElement(PolicyInclude.AXIS_SERVICE_POLICY, svcEffectivePolicy);
             }
         }
 
@@ -434,15 +538,35 @@ public class ProxyService {
         msgRcvr.setName(name);
         msgRcvr.setProxy(this);
 
-        iter = proxyService.getOperations();
+        Iterator iter = proxyService.getOperations();
         while (iter.hasNext()) {
             AxisOperation op = (AxisOperation) iter.next();
             op.setMessageReceiver(msgRcvr);
         }
 
         try {
-            auditInfo("Adding service " + name + " to the Axis2 configuration");
-            axisCfg.addService(proxyService);
+            proxyService.addParameter(
+                    SynapseConstants.SERVICE_TYPE_PARAM_NAME, SynapseConstants.PROXY_SERVICE_TYPE);
+            if (serviceGroup == null) {
+                auditInfo("Adding service " + name + " to the Axis2 configuration");
+                axisCfg.addService(proxyService);
+            } else {
+                auditInfo("Adding service " + name + " to the service group " + serviceGroup);
+                if (axisCfg.getServiceGroup(serviceGroup) == null) {
+                    // If the specified group does not exist we should create it
+                    AxisServiceGroup proxyServiceGroup = new AxisServiceGroup();
+                    proxyServiceGroup.setServiceGroupName(serviceGroup);
+                    proxyServiceGroup.setParent(axisCfg);
+                    // Add  the service to the new group and add the group the AxisConfiguration
+                    proxyServiceGroup.addService(proxyService);
+                    axisCfg.addServiceGroup(proxyServiceGroup);
+                } else {
+                    // Simply add the service to the existing group
+                    AxisServiceGroup proxyServiceGroup = axisCfg.getServiceGroup(serviceGroup);
+                    proxyService.setParent(proxyServiceGroup);
+                    proxyServiceGroup.addService(proxyService);
+                }
+            }
             this.setRunning(true);
         } catch (AxisFault axisFault) {
             try {
@@ -455,13 +579,23 @@ public class ProxyService {
             handleException("Error adding Proxy service to the Axis2 engine", axisFault);
         }
 
-        // todo: need to remove this and engage modules by looking at policies
+        // should Addressing be engaged on this service?
+        if (wsAddrEnabled) {
+            auditInfo("WS-Addressing is enabled for service : " + name);
+            try {
+                proxyService.engageModule(axisCfg.getModule(
+                    SynapseConstants.ADDRESSING_MODULE_NAME), axisCfg);
+            } catch (AxisFault axisFault) {
+                handleException("Error loading WS Addressing module on proxy service : " + name, axisFault);
+            }
+        }
+
         // should RM be engaged on this service?
         if (wsRMEnabled) {
             auditInfo("WS-Reliable messaging is enabled for service : " + name);
             try {
                 proxyService.engageModule(axisCfg.getModule(
-                    SynapseConstants.SANDESHA2_MODULE_NAME), axisCfg);
+                    SynapseConstants.RM_MODULE_NAME), axisCfg);
             } catch (AxisFault axisFault) {
                 handleException("Error loading WS RM module on proxy service : " + name, axisFault);
             }
@@ -472,7 +606,7 @@ public class ProxyService {
             auditInfo("WS-Security is enabled for service : " + name);
             try {
                 proxyService.engageModule(axisCfg.getModule(
-                    SynapseConstants.RAMPART_MODULE_NAME), axisCfg);
+                    SynapseConstants.SECURITY_MODULE_NAME), axisCfg);
             } catch (AxisFault axisFault) {
                 handleException("Error loading WS Sec module on proxy service : "
                         + name, axisFault);
@@ -483,6 +617,13 @@ public class ProxyService {
         return proxyService;
     }
 
+    private Policy getPolicyFromKey(String key, SynapseConfiguration synCfg) {
+
+        synCfg.getEntryDefinition(key);
+        return PolicyEngine.getPolicy(
+                SynapseConfigUtils.getStreamSource(synCfg.getEntry(key)).getInputStream());
+    }
+
     /**
      * Start the proxy service
      * @param synCfg the synapse configuration
@@ -490,7 +631,7 @@ public class ProxyService {
     public void start(SynapseConfiguration synCfg) {
         AxisConfiguration axisConfig = synCfg.getAxisConfiguration();
         if (axisConfig != null) {
-            
+
             Parameter param = axisConfig.getParameter(SynapseConstants.SYNAPSE_ENV);
             if (param != null && param.getValue() instanceof SynapseEnvironment)  {
                 SynapseEnvironment env = (SynapseEnvironment) param.getValue();
@@ -507,12 +648,14 @@ public class ProxyService {
                 auditWarn("Unable to find the SynapseEnvironment. " +
                     "Components of the proxy service may not be initialized");
             }
-            
-            axisConfig.getServiceForActivation(this.getName()).setActive(true);
+
+            AxisService as = axisConfig.getServiceForActivation(this.getName());
+            as.setActive(true);
+            axisConfig.notifyObservers(AxisEvent.SERVICE_START, as);
             this.setRunning(true);
             auditInfo("Started the proxy service : " + name);
         } else {
-            auditWarn("Unable to start proxy service : " + name + 
+            auditWarn("Unable to start proxy service : " + name +
                 ". Couldn't access Axis configuration");
         }
     }
@@ -539,9 +682,10 @@ public class ProxyService {
                 AxisService as = axisConfig.getService(this.getName());
                 if (as != null) {
                     as.setActive(false);
+                    axisConfig.notifyObservers(AxisEvent.SERVICE_STOP, as);
                 }
                 this.setRunning(false);
-                auditInfo("Started the proxy service : " + name);
+                auditInfo("Stopped the proxy service : " + name);
             } catch (AxisFault axisFault) {
                 handleException("Error stopping the proxy service : " + name, axisFault);
             }
@@ -617,7 +761,7 @@ public class ProxyService {
         parameters.put(name, value);
     }
 
-    public Map getParameterMap() {
+    public Map<String, Object> getParameterMap() {
         return this.parameters;
     }
 
@@ -657,7 +801,7 @@ public class ProxyService {
         this.wsdlKey = wsdlKey;
     }
 
-    public List getServiceLevelPolicies() {
+    public List<String> getServiceLevelPolicies() {
         return serviceLevelPolicies;
     }
 
@@ -665,6 +809,14 @@ public class ProxyService {
         this.serviceLevelPolicies.add(serviceLevelPolicy);
     }
 
+    public boolean isWsAddrEnabled() {
+        return wsAddrEnabled;
+    }
+
+    public void setWsAddrEnabled(boolean wsAddrEnabled) {
+        this.wsAddrEnabled = wsAddrEnabled;
+    }
+
     public boolean isWsRMEnabled() {
         return wsRMEnabled;
     }
@@ -696,25 +848,7 @@ public class ProxyService {
     public void setRunning(boolean running) {
         this.running = running;
     }
-
-    /**
-     * To check whether statistics should have collected or not
-     *
-     * @return Returns the int value that indicate statistics is enabled or not.
-     */
-    public int getStatisticsState() {
-        return statisticsState;
-    }
-
-    /**
-     * To set the statistics enable variable value
-     *
-     * @param statisticsState
-     */
-    public void setStatisticsState(int statisticsState) {
-        this.statisticsState = statisticsState;
-    }
-
+   
     /**
      * Returns the int value that indicate the tracing state
      *
@@ -727,7 +861,7 @@ public class ProxyService {
     /**
      * Set the tracing State variable
      *
-     * @param traceState
+     * @param traceState tracing state
      */
     public void setTraceState(int traceState) {
         this.traceState = traceState;
@@ -790,11 +924,11 @@ public class ProxyService {
     }
 
     public List getPinnedServers() {
-      return pinnedServers;
+        return pinnedServers;
     }
 
     public void setPinnedServers(List pinnedServers) {
-      this.pinnedServers = pinnedServers;
+        this.pinnedServers = pinnedServers;
     }
 
     public ResourceMap getResourceMap() {
@@ -805,4 +939,70 @@ public class ProxyService {
         this.resourceMap = resourceMap;
     }
 
+    public List<String> getInMessagePolicies() {
+        return inMessagePolicies;
+    }
+
+    public void setInMessagePolicies(List<String> inMessagePolicies) {
+        this.inMessagePolicies = inMessagePolicies;
+    }
+
+    public void addInMessagePolicy(String messagePolicy) {
+        this.inMessagePolicies.add(messagePolicy);
+    }
+
+    public List<String> getOutMessagePolicies() {
+        return outMessagePolicies;
+    }
+
+    public void setOutMessagePolicies(List<String> outMessagePolicies) {
+        this.outMessagePolicies = outMessagePolicies;
+    }
+
+    public void addOutMessagePolicy(String messagePolicy) {
+        this.outMessagePolicies.add(messagePolicy);
+    }
+
+    public List<PolicyInfo> getPolicies() {
+        return policies;
+    }
+
+    public void setPolicies(List<PolicyInfo> policies) {
+        this.policies = policies;
+    }
+
+    public void addPolicyInfo(PolicyInfo pi) {
+        this.policies.add(pi);
+    }
+
+    public void configure(AspectConfiguration aspectConfiguration) {
+        this.aspectConfiguration = aspectConfiguration;
+    }
+
+    public AspectConfiguration getAspectConfiguration() {
+        return aspectConfiguration;
+    }
+
+    public String getFileName() {
+        return fileName;
+    }
+
+    public void setFileName(String fileName) {
+        this.fileName = fileName;
+    }
+
+    public String getServiceGroup() {
+        return serviceGroup;
+    }
+
+    public void setServiceGroup(String serviceGroup) {
+        this.serviceGroup = serviceGroup;
+    }
+
+    @Override
+    public String toString() {
+        StringBuffer sb = new StringBuffer();
+        sb.append("[ Proxy Service [ Name : ").append(name).append(" ] ]");
+        return sb.toString();
+    }
 }
