@@ -19,10 +19,9 @@
 
 package org.apache.synapse.core.axis2;
 
-import org.apache.axiom.soap.SOAPEnvelope;
-import org.apache.axiom.soap.SOAPFault;
-import org.apache.axiom.soap.SOAPFaultReason;
+import org.apache.axiom.om.OMException;
 import org.apache.axis2.AxisFault;
+import org.apache.axis2.Constants;
 import org.apache.axis2.addressing.AddressingConstants;
 import org.apache.axis2.addressing.EndpointReference;
 import org.apache.axis2.addressing.RelatesTo;
@@ -30,15 +29,22 @@ import org.apache.axis2.client.Options;
 import org.apache.axis2.client.async.AxisCallback;
 import org.apache.axis2.context.MessageContext;
 import org.apache.axis2.engine.MessageReceiver;
+import org.apache.axis2.wsdl.WSDLConstants;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.sandesha2.client.SandeshaClientConstants;
 import org.apache.synapse.FaultHandler;
 import org.apache.synapse.SynapseConstants;
 import org.apache.synapse.SynapseException;
+import org.apache.synapse.ServerContextInformation;
+import org.apache.synapse.aspects.statistics.ErrorLogFactory;
+import org.apache.synapse.aspects.statistics.StatisticsReporter;
+import org.apache.synapse.config.SynapseConfigUtils;
 import org.apache.synapse.config.SynapseConfiguration;
 import org.apache.synapse.endpoints.Endpoint;
+import org.apache.synapse.endpoints.dispatch.Dispatcher;
 import org.apache.synapse.transport.nhttp.NhttpConstants;
+import org.apache.synapse.util.ResponseAcceptEncodingProcessor;
 
 import java.util.*;
 
@@ -55,34 +61,38 @@ public class SynapseCallbackReceiver implements MessageReceiver {
     private static final Log log = LogFactory.getLog(SynapseCallbackReceiver.class);
 
     /** This is the synchronized callbackStore that maps outgoing messageID's to callback objects */
-    private Map callbackStore;  // this will be made thread safe within the constructor
+    private final Map<String, AxisCallback> callbackStore;  // will made thread safe in the constructor
 
     /**
      * Create the *single* instance of this class that would be used by all anonymous services
      * used for outgoing messaging.
      * @param synCfg the Synapse configuration
+     * @param contextInformation server runtime information
      */
-    public SynapseCallbackReceiver(SynapseConfiguration synCfg) {
+    public SynapseCallbackReceiver(SynapseConfiguration synCfg,
+                                   ServerContextInformation contextInformation) {
 
-        callbackStore = Collections.synchronizedMap(new HashMap());
+        callbackStore = Collections.synchronizedMap(new HashMap<String, AxisCallback>());
 
         // create the Timer object and a TimeoutHandler task
-        TimeoutHandler timeoutHandler = new TimeoutHandler(callbackStore);
+        TimeoutHandler timeoutHandler = new TimeoutHandler(callbackStore, contextInformation);
         
         Timer timeOutTimer = synCfg.getSynapseTimer();
-        long timeoutHandlerInterval = SynapseConstants.DEFAULT_TIMEOUT_HANDLER_INTERVAL;
-        try {
-            timeoutHandlerInterval = Long.parseLong(
-                System.getProperty(SynapseConstants.TIMEOUT_HANDLER_INTERVAL));
-        } catch (Exception ignore) {}
+        long timeoutHandlerInterval = SynapseConfigUtils.getTimeoutHandlerInterval();
 
         // schedule timeout handler to run every n seconds (n : specified or defaults to 15s)
         timeOutTimer.schedule(timeoutHandler, 0, timeoutHandlerInterval);
     }
 
+    public int getCallbackCount() {
+        return callbackStore.size();
+    }
 
     public void addCallback(String MsgID, AxisCallback callback) {
         callbackStore.put(MsgID, callback);
+        if (log.isDebugEnabled()) {
+            log.debug("Callback added. Total callbacks waiting for : " + callbackStore.size());
+        }
     }
 
     /**
@@ -111,7 +121,11 @@ public class SynapseCallbackReceiver implements MessageReceiver {
         }
 
         if (messageID != null) {
-            AxisCallback callback = (AxisCallback) callbackStore.remove(messageID);
+            AxisCallback callback = callbackStore.remove(messageID);
+            if (log.isDebugEnabled()) {
+                log.debug("Callback removed for request message id : " + messageID +
+                        ". Pending callbacks count : " + callbackStore.size());
+            }
 
             RelatesTo[] relates = messageCtx.getRelationships();
             if (relates != null && relates.length > 1) {
@@ -122,12 +136,12 @@ public class SynapseCallbackReceiver implements MessageReceiver {
             }
             
             if (callback != null) {
-                handleMessage(messageCtx, ((AsyncCallback) callback).getSynapseOutMsgCtx());
+                handleMessage(messageID, messageCtx, ((AsyncCallback) callback).getSynapseOutMsgCtx());
                 
             } else {
                 // TODO invoke a generic synapse error handler for this message
                 log.warn("Synapse received a response for the request with message Id : " +
-                    messageID + " But a callback is not registered (anymore) to process this response");
+                        messageID + " But a callback is not registered (anymore) to process this response");
             }
 
         } else if (!messageCtx.isPropertyTrue(NhttpConstants.SC_ACCEPTED)){
@@ -139,46 +153,53 @@ public class SynapseCallbackReceiver implements MessageReceiver {
     /**
      * Handle the response or error (during a failed send) message received for an outgoing request
      *
+     * @param messageID        Request message ID
      * @param response         the Axis2 MessageContext that has been received and has to be handled
      * @param synapseOutMsgCtx the corresponding (outgoing) Synapse MessageContext for the above
      *                         Axis2 MC, that holds Synapse specific information such as the error
      *                         handler stack and local properties etc.
-     * @throws AxisFault 
+     * @throws AxisFault       if the message cannot be processed
      */
-    private void handleMessage(MessageContext response,
+    private void handleMessage(String messageID ,MessageContext response,
         org.apache.synapse.MessageContext synapseOutMsgCtx) throws AxisFault {
 
-        Object o = response.getProperty(NhttpConstants.SENDING_FAULT);
+        Object o = response.getProperty(SynapseConstants.SENDING_FAULT);
         if (o != null && Boolean.TRUE.equals(o)) {
 
+            StatisticsReporter.reportFaultForAll(synapseOutMsgCtx,
+                    ErrorLogFactory.createErrorLog(response));
             // there is a sending fault. propagate the fault to fault handlers.
 
             Stack faultStack = synapseOutMsgCtx.getFaultStack();
             if (faultStack != null && !faultStack.isEmpty()) {
-                SOAPEnvelope envelope = response.getEnvelope();
-                if (envelope != null) {
-                    SOAPFault fault = envelope.getBody().getFault();
-                    if (fault != null) {
-                        Exception e = fault.getException();
-                        if (e == null) {
-                            e = new Exception(fault.toString());
-                        }
-                        // set an error code to the message context, so that error sequences can
-                        // filter using that property to determine the cause of error
-                        synapseOutMsgCtx.setProperty(SynapseConstants.ERROR_CODE,
-                            SynapseConstants.SENDING_FAULT);
-                        SOAPFaultReason faultReason = fault.getReason();
-                        if (faultReason != null) {
-                            synapseOutMsgCtx.setProperty(SynapseConstants.ERROR_MESSAGE,
-                                    faultReason.getText());
-                        }
-                        if (fault.getException() != null) {
-                            synapseOutMsgCtx.setProperty(SynapseConstants.ERROR_EXCEPTION,
-                                    fault.getException());
-                        }
-                        ((FaultHandler) faultStack.pop()).handleFault(synapseOutMsgCtx, e);
-                    }
-                }
+
+                // if we have access to the full synapseOutMsgCtx.getEnvelope(), then let
+                // it flow with the error details. Else, replace its envelope with the
+                // fault envelope
+                try {
+                    synapseOutMsgCtx.getEnvelope().build();
+                } catch (OMException x) {
+                    synapseOutMsgCtx.setEnvelope(response.getEnvelope());
+                  }
+
+                Exception e = (Exception) response.getProperty(SynapseConstants.ERROR_EXCEPTION);
+
+                synapseOutMsgCtx.setProperty(SynapseConstants.SENDING_FAULT, Boolean.TRUE);
+                synapseOutMsgCtx.setProperty(SynapseConstants.ERROR_CODE,
+                    response.getProperty(SynapseConstants.ERROR_CODE));
+                synapseOutMsgCtx.setProperty(SynapseConstants.ERROR_MESSAGE,
+                    response.getProperty(SynapseConstants.ERROR_MESSAGE));
+                synapseOutMsgCtx.setProperty(SynapseConstants.ERROR_DETAIL,
+                    response.getProperty(SynapseConstants.ERROR_DETAIL));
+                synapseOutMsgCtx.setProperty(SynapseConstants.ERROR_EXCEPTION, e);
+
+                if (log.isDebugEnabled()) {
+                    log.debug("[Failed Request Message ID : " + messageID + "]" +
+                            " [New to be Retried Request Message ID : " +
+                            synapseOutMsgCtx.getMessageID() + "]");
+                }                   
+
+                ((FaultHandler) faultStack.pop()).handleFault(synapseOutMsgCtx, null);
             }
 
         } else {
@@ -188,7 +209,8 @@ public class SynapseCallbackReceiver implements MessageReceiver {
             Stack faultStack = synapseOutMsgCtx.getFaultStack();
             if (faultStack !=null && !faultStack.isEmpty()
                 && faultStack.peek() instanceof Endpoint) {
-                faultStack.pop();
+                Endpoint successfulEndpoint = (Endpoint) faultStack.pop();
+                successfulEndpoint.onSuccess();
             }
             if (log.isDebugEnabled()) {
                 log.debug("Synapse received an asynchronous response message");
@@ -200,8 +222,8 @@ public class SynapseCallbackReceiver implements MessageReceiver {
                         (response.getWSAAction() != null ? response.getWSAAction() : "null"));
                 String[] cids = response.getAttachmentMap().getAllContentIDs();
                 if (cids != null && cids.length > 0) {
-                    for (int i = 0; i < cids.length; i++) {
-                        log.debug("Attachment : " + cids[i]);
+                    for (String cid : cids) {
+                        log.debug("Attachment : " + cid);
                     }
                 }
                 log.debug("Body : \n" + response.getEnvelope());
@@ -209,11 +231,13 @@ public class SynapseCallbackReceiver implements MessageReceiver {
             MessageContext axisOutMsgCtx =
                     ((Axis2MessageContext) synapseOutMsgCtx).getAxis2MessageContext();
 
+            //Processes 'Accept-Encoding'
+            ResponseAcceptEncodingProcessor.process(response, axisOutMsgCtx);
+
             response.setServiceContext(null);
             response.setOperationContext(axisOutMsgCtx.getOperationContext());
-            response.getAxisMessage().setParent(
-                axisOutMsgCtx.getOperationContext().getAxisOperation());
-            response.setAxisService(axisOutMsgCtx.getAxisService());
+            response.setAxisMessage(axisOutMsgCtx.getAxisOperation().getMessage(
+                    WSDLConstants.MESSAGE_LABEL_OUT_VALUE));
 
             // set properties on response
             response.setServerSide(true);
@@ -240,9 +264,29 @@ public class SynapseCallbackReceiver implements MessageReceiver {
                         org.apache.axis2.Constants.VALUE_TRUE);
             }
 
-            // clear the message type property thats used by the message formatter later on
-            // to decide whether to write as SOAP/POX etc..
-            response.removeProperty(org.apache.axis2.Constants.Configuration.MESSAGE_TYPE);
+            // when axis2 receives a soap message without addressing headers it users
+            // DISABLE_ADDRESSING_FOR_OUT_MESSAGES property to keep it and hence avoid addressing
+            // headers on the response. this causes a problem for synapse if the original message
+            // it receivs (from client) has addressing and the synaspse service invocation has not
+            // engage addressing. in this case when synapse receives the response from the server
+            // addessing In handler dissable addressing since that response does not have addressing
+            // headers. synapse sends the response to its orignal client using the same message
+            // context. Then this response does not have addressing headers since it already
+            // disable. to avoid this we need to set the DISABLE_ADDRESSING_FOR_OUT_MESSAGES
+            // property state to original state.
+            if (axisOutMsgCtx.getProperty(
+                    AddressingConstants.DISABLE_ADDRESSING_FOR_OUT_MESSAGES) != null) {
+                
+                response.setProperty(AddressingConstants.DISABLE_ADDRESSING_FOR_OUT_MESSAGES,
+                        axisOutMsgCtx.getProperty(
+                                AddressingConstants.DISABLE_ADDRESSING_FOR_OUT_MESSAGES));
+            } else {
+                response.removeProperty(AddressingConstants.DISABLE_ADDRESSING_FOR_OUT_MESSAGES);
+            }
+
+             // copy the message type property thats used by the out message to the response message
+            response.setProperty(org.apache.axis2.Constants.Configuration.MESSAGE_TYPE,
+                axisOutMsgCtx.getProperty(org.apache.axis2.Constants.Configuration.MESSAGE_TYPE));
             
             // compare original received message (axisOutMsgCtx) soap version with the response
             // if they are different change to original version 
@@ -259,6 +303,13 @@ public class SynapseCallbackReceiver implements MessageReceiver {
                         new RelatesTo[]{new RelatesTo(axisOutMsgCtx.getMessageID())});
             }
 
+            response.setReplyTo(axisOutMsgCtx.getReplyTo());
+            response.setFaultTo(axisOutMsgCtx.getFaultTo());
+
+            if (axisOutMsgCtx.isPropertyTrue(NhttpConstants.IGNORE_SC_ACCEPTED)) {
+                response.setProperty(NhttpConstants.FORCE_SC_ACCEPTED, Constants.VALUE_TRUE);
+            }
+
             // create the synapse message context for the response
             Axis2MessageContext synapseInMessageContext =
                     new Axis2MessageContext(
@@ -272,14 +323,22 @@ public class SynapseCallbackReceiver implements MessageReceiver {
             synapseInMessageContext.setTracingState(synapseOutMsgCtx.getTracingState());
 
             // set the properties of the original MC to the new MC
-            Iterator iter = synapseOutMsgCtx.getPropertyKeySet().iterator();
 
-            while (iter.hasNext()) {
-                Object key = iter.next();
+            for (Object key : synapseOutMsgCtx.getPropertyKeySet()) {
                 synapseInMessageContext.setProperty(
                         (String) key, synapseOutMsgCtx.getProperty((String) key));
+            }            
+
+            // If this response is related to session affinity endpoints -Server initiated session
+            Dispatcher dispatcher =
+                    (Dispatcher) synapseOutMsgCtx.getProperty(
+                            SynapseConstants.PROP_SAL_ENDPOINT_CURRENT_DISPATCHER);
+            if (dispatcher != null && dispatcher.isServerInitiatedSession()) {
+                dispatcher.updateSession(synapseInMessageContext);
             }
 
+            StatisticsReporter.reportForAllOnResponseReceived(synapseInMessageContext);
+            
             // send the response message through the synapse mediation flow
             try {
                 synapseOutMsgCtx.getEnvironment().injectMessage(synapseInMessageContext);
@@ -309,8 +368,7 @@ public class SynapseCallbackReceiver implements MessageReceiver {
         int insertPos = 0;
         RelatesTo[] newRelates = new RelatesTo[relates.length];
 
-        for (int i = 0; i < relates.length; i++) {
-            RelatesTo current = relates[i];
+        for (RelatesTo current : relates) {
             boolean found = false;
             for (int j = 0; j < newRelates.length && j < insertPos; j++) {
                 if (newRelates[j].equals(current) ||
